/*  detect.h
 *
 *  Header file for detect.c
 */

/*
 *  xdemorse: An application to decode Morse signals to text
 *
 *  Copyright (C) 2001  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#ifndef DETECT_H
#define DETECT_H	1

#include <math.h>
#include "common.h"

#define CYCLES_PER_FRAG		2	/* Cycles of signal/signal fragment  */
#define SIGNAL_SCALE		150 /* Scale factor to fit signal in scope */
#define DETECT_SCALE		250 /* Scale factor to fit detector in scope */
#define THRESH_SCALE		300 /* Factor to sacle slope bkgnd to threshold */
#define SLOPE_AVERAGE	   1000 /* Slope averaging convergence factor */

/* Signal Detector slope status */
#define SLOPE_POSITIVE		1
#define SLOPE_NEGATIVE		2
#define SLOPE_ZERO			3

/* Buffers and data needed by tone detector */
struct detector_data
{
  int
	frag_len,
	samples_buff_len, /* Length of DSP samples buffer */
	samples_buff_idx; /* Sample buffer index */

  /* Circular signal level buffer and index */
  int *sig_level_buff;
  int sig_level_idx;

  /* Circular slope status buffer and index */
  char *slope_status_buff;
  int slope_status_idx;

  /* Variables for the Goertzel algorithm */
  double cosw, sinw, coeff;

  /* Circular signal samples buffer for Goertzel detector */
  short *samples_buff;

} detector_data;

#endif

